Package: incidence
Type: Package
Title: Compute, Handle, Plot and Model Incidence of Dated Events
Version: 1.7.6
Authors@R: c(
  person("Thibaut", "Jombart", role = c("aut"), email = "thibautjombart@gmail.com"), 
  person("Zhian N.", "Kamvar", role = "aut", email = "zkamvar@gmail.com",
         comment = c(ORCID = "0000-0003-1458-7108")),
  person("Rich", "FitzJohn", role = "aut", email = "rich.fitzjohn@gmail.com"),
  person("Tim", "Taylor", role = "cre", email = "tim.taylor@hiddenelephants.co.uk",
         comment = c(ORCID = "0000-0002-8587-7113")),
  person("Jun", "Cai", role = "ctb", email = "cai-j12@mails.tsinghua.edu.cn", 
         comment = c(ORCID = "0000-0001-9495-1226")), 
  person("Sangeeta", "Bhatia", role = "ctb", email = "sangeetabhatia03@gmail.com"),
  person("Jakob", "Schumacher", role = "ctb"),
  person("Juliet R.C.", "Pulliam", role = "ctb", email = "pulliam@sun.ac.za",
         comment = c(ORCID = "0000-0003-3314-8223"))
  )
Description: Provides functions and classes to compute, handle and visualise
  incidence from dated events for a defined time interval. Dates can be provided
  in various standard formats. The class 'incidence' is used to store computed
  incidence and can be easily manipulated, subsetted, and plotted. In addition,
  log-linear models can be fitted to 'incidence' objects using 'fit'. This
  package is part of the RECON (<https://www.repidemicsconsortium.org/>) toolkit
  for outbreak analysis.
Encoding: UTF-8
License: MIT + file LICENSE
URL: https://www.repidemicsconsortium.org/incidence/
BugReports: https://github.com/reconhub/incidence/issues
RoxygenNote: 7.3.2
Imports: ggplot2 (>= 3.3.2), aweek (>= 0.2.0)
Suggests: magrittr, outbreaks, testthat, vdiffr, knitr, rmarkdown,
        scales, cowplot
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-06-19 21:23:53 UTC; tim
Author: Thibaut Jombart [aut],
  Zhian N. Kamvar [aut] (ORCID: <https://orcid.org/0000-0003-1458-7108>),
  Rich FitzJohn [aut],
  Tim Taylor [cre] (ORCID: <https://orcid.org/0000-0002-8587-7113>),
  Jun Cai [ctb] (ORCID: <https://orcid.org/0000-0001-9495-1226>),
  Sangeeta Bhatia [ctb],
  Jakob Schumacher [ctb],
  Juliet R.C. Pulliam [ctb] (ORCID:
    <https://orcid.org/0000-0003-3314-8223>)
Maintainer: Tim Taylor <tim.taylor@hiddenelephants.co.uk>
Repository: CRAN
Date/Publication: 2025-06-19 21:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 06:43:53 UTC; unix
