# Generated by roxygen2: do not edit by hand

S3method("[",incidence)
S3method("group_names<-",default)
S3method("group_names<-",incidence)
S3method(as.data.frame,incidence)
S3method(as.incidence,data.frame)
S3method(as.incidence,matrix)
S3method(as.incidence,numeric)
S3method(cumulate,default)
S3method(cumulate,incidence)
S3method(dim,incidence)
S3method(get_counts,incidence)
S3method(get_dates,default)
S3method(get_dates,incidence)
S3method(get_fit,incidence_fit)
S3method(get_fit,incidence_fit_list)
S3method(get_info,incidence_fit)
S3method(get_info,incidence_fit_list)
S3method(get_interval,default)
S3method(get_interval,incidence)
S3method(get_n,default)
S3method(get_n,incidence)
S3method(get_timespan,default)
S3method(get_timespan,incidence)
S3method(group_names,default)
S3method(group_names,incidence)
S3method(incidence,Date)
S3method(incidence,POSIXt)
S3method(incidence,character)
S3method(incidence,default)
S3method(incidence,integer)
S3method(incidence,numeric)
S3method(plot,incidence)
S3method(plot,incidence_fit)
S3method(plot,incidence_fit_list)
S3method(print,incidence)
S3method(print,incidence_fit)
S3method(print,incidence_fit_list)
S3method(subset,incidence)
export("group_names<-")
export(add_incidence_fit)
export(as.incidence)
export(bootstrap)
export(cumulate)
export(estimate_peak)
export(find_peak)
export(fit)
export(fit_optim_split)
export(get_counts)
export(get_dates)
export(get_fit)
export(get_info)
export(get_interval)
export(get_n)
export(get_timespan)
export(group_names)
export(incidence)
export(incidence_pal1)
export(incidence_pal1_dark)
export(incidence_pal1_light)
export(make_breaks)
export(pool)
export(scale_x_incidence)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,plot)
importFrom(stats,as.ts)
importFrom(utils,head)
importFrom(utils,tail)
